from Products.CMFCore.utils import getToolByName

from collective.smsauthenticator import __title__

INSTALL = 'profile-%s:default' % __title__
UNINSTALL = 'profile-%s:uninstall' % __title__


def uninstall(portal, reinstall=False):
    """ Uninstall this product.
        This external method is need, because portal_quickinstaller doens't
        know what GenericProfile profile to apply when uninstalling a product.
    """
    setup_tool = getToolByName(portal, 'portal_setup')
    if reinstall:
        return "Ran all reinstall steps."
    else:
        setup_tool.runAllImportStepsFromProfile(UNINSTALL)
        return "Ran all uninstall steps."



__title__ = 'collective.smsauthenticator'
__version__ = '0.3.1'
__build__ = 0x00000d
__author__ = 'Goldmund, Wyldebeast & Wunderliebe <info@gw20e.com>'
__copyright__ = 'Goldmund, Wyldebeast & Wunderliebe'
__license__ = 'GPL 2.0/LGPL 2.1'

from zope.i18nmessageid import MessageFactory

from Products.PluggableAuthService.PluggableAuthService import registerMultiPlugin

from collective.smsauthenticator.pas_plugin import (
    SMSAuthenticatorPlugin, addSMSAuthenticatorPlugin, manage_addSMSAuthenticatorPluginForm
    )

_ = MessageFactory('collective.smsauthenticator')

def initialize(context):
    """
    Initializer called when used as a Zope 2 product.
    """
    registerMultiPlugin(SMSAuthenticatorPlugin.meta_type) # Add to PAS menu
    context.registerClass(
        SMSAuthenticatorPlugin,
        constructors = (manage_addSMSAuthenticatorPluginForm, addSMSAuthenticatorPlugin),
        visibility = None
        )



from zope.i18nmessageid import MessageFactory

_ = MessageFactory('collective.smsauthenticator')

def get_app_links(context):
    """
    Gets SMS authenticator app links.
    """
    return {
        # Settings URL
        'settings_url': '{0}/{1}'.format(context.absolute_url(), '@@sms-authenticator-settings'),
        'settings_text': _("SMS Authenticator configuration"),
        # Enable URL
        'enable_url': '{0}/{1}'.format(context.absolute_url(), '@@sms-authenticator-enable-for-all-users'),
        'enable_text': _("Enable two-step verification for all users"),
        # Disable URL
        'disable_url': '{0}/{1}'.format(context.absolute_url(), '@@sms-authenticator-disable-for-all-users'),
        'disable_text':  _("Disable two-step verification for all users"),
        # See all IPs URL
        'show_all_user_ips_url': '{0}/{1}'.format(context.absolute_url(), '@@sms-authenticator-show-all-user-ips'),
        'show_all_user_ips_text': _("Show all user IPs"),
        # See unique IPs URL
        'show_unique_user_ips_url': '{0}/{1}'.format(context.absolute_url(), '@@sms-authenticator-show-unique-user-ips'),
        'show_unique_user_ips_text': _("Show unique user IPs"),
    }



import datetime

from zope.i18nmessageid import MessageFactory

from plone import api

from Products.Five import BrowserView

from collective.smsauthenticator.browser.helpers import get_app_links

_ = MessageFactory('collective.smsauthenticator')

class ListUserIPs(BrowserView):
    """
    List user IPS, generic view
    """

    def _get_user_unique_ips(self, user, with_username=False):
        """
        Gets unique IPs of a user.

        :param Products.PlonePAS.tools.memberdata user:
        :return: Set of user unique IPs.
        """
        unique_ips = set()
        raw_ips = user.getProperty('ips', '')

        if raw_ips:
            all_ips = raw_ips.split('\n')

            for ip_line in all_ips:
                if ip_line:
                    try:
                        ip, visit = ip_line.split(',', 2)
                        try:
                            visit = datetime.datetime.fromtimestamp(float(visit))
                        except Exception as e:
                            pass
                        if with_username:
                            data = (ip, user.getId())
                        else:
                            data = ip
                        unique_ips.add(ip)
                    except Exception as e:
                        pass

        return unique_ips

    def _get_user_ips(self, user, with_username=False):
        """
        Gets all IPs of a user.

        :param Products.PlonePAS.tools.memberdata user:
        :return: List of user IPs.
        """
        unique_ips = []
        raw_ips = user.getProperty('ips', '')

        if raw_ips:
            all_ips = raw_ips.split('\n')

            for ip_line in all_ips:
                if ip_line:
                    try:
                        ip, visit = ip_line.split(',', 2)
                        try:
                            visit = datetime.datetime.fromtimestamp(float(visit))
                        except Exception as e:
                            pass
                        if with_username:
                            data = (ip, visit, user.getId())
                        else:
                            data = (ip, visit)
                        unique_ips.append(data)
                    except Exception as e:
                        pass

        return unique_ips

    def get_app_links(self):
        """ View function to pass function along to actual helper """
        return get_app_links(self.context)

class ListUniqueUserIPs(ListUserIPs):
    """
    BrowserView for List unique user IPs.
    """

    def get_unique_ips(self):
        users = api.user.get_users()
        ips = set()
        for user in users:
            ips.update(self._get_user_unique_ips(user, with_username=False))

        return ips

class ListAllUserIPs(ListUserIPs):
    """
    BrowserView for List all user IPs.
    """

    def get_all_ips(self):

        users = api.user.get_users()
        ips = []
        for user in users:
            ips += self._get_user_ips(user, with_username=True)

        return ips
